import os
import subprocess
import sys
from pathlib import Path
from unittest import mock

from django.test import TestCase

from core import release


class PyPITokenTests(TestCase):
    def test_publish_uses_token_when_password_missing(self):
        creds = release.Credentials(
            token="pypi-token", username="ignored", password=None
        )
        with (
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
            mock.patch("core.release.subprocess.run") as run,
        ):
            run.return_value.returncode = 0
            run.return_value.stdout = ""
            run.return_value.stderr = ""
            release.publish(version="0.1.1", creds=creds)
        commands = [call.args[0] for call in run.call_args_list]
        twine_cmd = next(cmd for cmd in commands if "twine" in cmd)
        assert "__token__" in twine_cmd
        assert "pypi-token" in twine_cmd
        assert "ignored" not in twine_cmd
        assert ["git", "tag", "v0.1.1"] in commands
        assert ["git", "push", "origin", "v0.1.1"] in commands

    def test_publish_prefers_profile_over_env(self):
        profile = release.Credentials(token="profile-token")
        env = {
            "PYPI_API_TOKEN": "env-token",
            "PYPI_USERNAME": "env-user",
            "PYPI_PASSWORD": "env-pass",
        }
        with (
            mock.patch.dict(os.environ, env, clear=False),
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
            mock.patch("core.release.subprocess.run") as run,
            mock.patch("core.release._manager_credentials", return_value=profile),
        ):
            run.return_value.returncode = 0
            run.return_value.stdout = ""
            run.return_value.stderr = ""
            release.publish(version="0.1.1")
        commands = [call.args[0] for call in run.call_args_list]
        twine_cmd = next(cmd for cmd in commands if "twine" in cmd)
        assert "__token__" in twine_cmd
        assert "profile-token" in twine_cmd
        assert "env-user" not in twine_cmd
        assert "env-pass" not in twine_cmd
        assert "env-token" not in twine_cmd
        assert ["git", "tag", "v0.1.1"] in commands
        assert ["git", "push", "origin", "v0.1.1"] in commands

    def test_publish_raises_when_dist_missing(self):
        with (
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=False),
        ):
            with self.assertRaises(release.ReleaseError) as exc:
                release.publish(version="0.1.1")
        assert str(exc.exception) == "dist directory not found"

    def test_publish_raises_when_dist_empty(self):
        with (
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(release.Path, "glob", return_value=[]),
        ):
            with self.assertRaises(release.ReleaseError) as exc:
                release.publish(version="0.1.1")
        assert str(exc.exception) == "dist directory is empty"

    def test_publish_raises_when_credentials_missing(self):
        with (
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
        ):
            with self.assertRaises(release.ReleaseError) as exc:
                release.publish(version="0.1.1", creds=release.Credentials())
        assert str(exc.exception) == "Missing PyPI credentials"

    def test_publish_raises_when_version_already_available(self):
        response = mock.Mock()
        response.ok = True
        response.json.return_value = {"releases": {"0.1.1": [{}]}}

        primary = release.RepositoryTarget(
            name="PyPI",
            verify_availability=True,
            credentials=release.Credentials(token="pypi-token"),
        )

        with (
            mock.patch("core.release.network_available", return_value=True),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
            mock.patch("requests.get", return_value=response) as request_get,
            mock.patch("core.release.subprocess.run") as run,
        ):
            run.return_value.returncode = 0
            run.return_value.stdout = ""
            run.return_value.stderr = ""
            with self.assertRaises(release.ReleaseError) as exc:
                release.publish(version="0.1.1", repositories=[primary])

        assert str(exc.exception) == "Version 0.1.1 already on PyPI"
        request_get.assert_called_once_with("https://pypi.org/pypi/arthexis/json")
        response.json.assert_called_once()

    def test_publish_skips_availability_check_when_disabled(self):
        response = mock.Mock()
        response.ok = True
        response.json.return_value = {"releases": {"0.1.1": [{}]}}

        primary = release.RepositoryTarget(
            name="PyPI",
            verify_availability=False,
            credentials=release.Credentials(token="pypi-token"),
        )

        with (
            mock.patch("core.release.network_available", return_value=True),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
            mock.patch("requests.get", return_value=response) as request_get,
            mock.patch("core.release.subprocess.run") as run,
        ):
            run.return_value.returncode = 0
            run.return_value.stdout = ""
            run.return_value.stderr = ""
            uploaded = release.publish(version="0.1.1", repositories=[primary])

        assert request_get.call_count == 0
        assert uploaded == ["PyPI"]

    def test_publish_supports_multiple_repositories(self):
        primary = release.RepositoryTarget(
            name="PyPI",
            verify_availability=True,
            credentials=release.Credentials(token="pypi-token"),
        )
        secondary = release.RepositoryTarget(
            name="GitHub Packages",
            repository_url="https://upload.github.com/pypi/",
            credentials=release.Credentials(
                username="octocat", password="gh-token"
            ),
        )
        package = release.Package(
            name="pkg",
            description="desc",
            author="author",
            email="author@example.com",
            python_requires=">=3.10",
            license="GPL",
        )

        def fake_run(cmd, capture_output=False, text=False, check=True):
            return subprocess.CompletedProcess(cmd, 0, stdout="", stderr="")

        with (
            mock.patch("core.release.network_available", return_value=False),
            mock.patch.object(release.Path, "exists", return_value=True),
            mock.patch.object(
                release.Path, "glob", return_value=[Path("dist/fake.whl")]
            ),
            mock.patch("core.release.subprocess.run", side_effect=fake_run) as run,
        ):
            release.publish(
                package=package,
                version="0.1.1",
                repositories=[primary, secondary],
            )

        commands = [call.args[0] for call in run.call_args_list]
        twine_commands = [cmd for cmd in commands if "twine" in cmd]
        assert len(twine_commands) == 2
        assert twine_commands[0][:4] == [sys.executable, "-m", "twine", "upload"]
        assert "--repository-url" not in twine_commands[0]
        assert "--repository-url" in twine_commands[1]
        assert "https://upload.github.com/pypi/" in twine_commands[1]



from __future__ import annotations

from pathlib import Path
import tempfile
from unittest import mock

from django.test import SimpleTestCase, override_settings
from django.utils import timezone

from core import system


class UpgradeReportTests(SimpleTestCase):
    def test_build_auto_upgrade_report_reads_files(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            base = Path(tmpdir)
            locks_dir = base / "locks"
            logs_dir = base / "logs"
            locks_dir.mkdir()
            logs_dir.mkdir()

            (locks_dir / "auto_upgrade.lck").write_text("latest", encoding="utf-8")
            (locks_dir / "auto_upgrade_skip_revisions.lck").write_text(
                "abc123\n\nxyz789\n",
                encoding="utf-8",
            )
            (logs_dir / "auto-upgrade.log").write_text(
                "2024-01-01T00:00:00+00:00 first run\n"
                "2024-01-01T01:00:00+00:00 second run\n",
                encoding="utf-8",
            )

            schedule_stub = {
                "available": False,
                "configured": False,
                "enabled": False,
                "one_off": False,
                "queue": "",
                "schedule": "",
                "start_time": "",
                "last_run_at": "",
                "next_run": "",
                "total_run_count": 0,
                "description": "",
                "expires": "",
                "task": "",
                "name": system.AUTO_UPGRADE_TASK_NAME,
                "error": "",
            }

            with override_settings(BASE_DIR=str(base)):
                with mock.patch(
                    "core.system._load_auto_upgrade_schedule",
                    return_value=schedule_stub,
                ):
                    report = system._build_auto_upgrade_report(limit=10)

        self.assertTrue(report["settings"]["enabled"])
        self.assertTrue(report["settings"]["is_latest"])
        self.assertEqual(report["settings"]["mode"], "latest")
        self.assertEqual(
            report["settings"]["skip_revisions"],
            ["abc123", "xyz789"],
        )
        self.assertEqual(
            [entry["message"] for entry in report["log_entries"]],
            ["first run", "second run"],
        )
        self.assertFalse(report["log_error"])
        self.assertTrue(report["settings"]["log_path"].endswith("auto-upgrade.log"))

    def test_load_auto_upgrade_schedule_uses_task_metadata(self):
        class DummySchedule:
            def __str__(self) -> str:
                return "every 5 minutes"

        class DummyTask:
            def __init__(self):
                self.enabled = True
                self.one_off = False
                self.queue = "default"
                self.total_run_count = 7
                self.description = "Auto-upgrade"
                self.task = "core.tasks.check_github_updates"
                self.name = system.AUTO_UPGRADE_TASK_NAME
                self.start_time = timezone.now()
                self.last_run_at = timezone.now()
                self.expires = None
                self._schedule = DummySchedule()

            @property
            def schedule(self):
                return self._schedule

        dummy_task = DummyTask()
        expected_start = system._format_timestamp(dummy_task.start_time)
        expected_last_run = system._format_timestamp(dummy_task.last_run_at)

        with mock.patch(
            "core.system._get_auto_upgrade_periodic_task",
            return_value=(dummy_task, True, ""),
        ), mock.patch(
            "core.system._auto_upgrade_next_check",
            return_value="Soon",
        ):
            info = system._load_auto_upgrade_schedule()

        self.assertTrue(info["available"])
        self.assertTrue(info["configured"])
        self.assertTrue(info["enabled"])
        self.assertEqual(info["schedule"], "every 5 minutes")
        self.assertEqual(info["next_run"], "Soon")
        self.assertEqual(info["total_run_count"], 7)
        self.assertEqual(info["task"], dummy_task.task)
        self.assertEqual(info["name"], dummy_task.name)
        self.assertEqual(info["start_time"], expected_start)
        self.assertEqual(info["last_run_at"], expected_last_run)
